tree grammar GroundAtomWalker;


options
{
	tokenVocab = GroundAtom;
	ASTLabelType = CommonTree;
}

@header
{
	package edu.cs.ai.kreator.parser.groundatom;

	import org.apache.log4j.Logger;

	import edu.cs.ai.kreator.exceptions.*;
	import edu.cs.ai.kreator.exceptions.runtime.*;
	import edu.cs.ai.kreator.logic.*;
	import edu.cs.ai.kreator.logic.LanguageFactory.*;
 	import edu.cs.ai.kreator.logic.VariableFactory.*;
	import edu.cs.ai.kreator.models.*;

	import edu.cs.ai.kreator.logic.Literal.*;

	/*	Werden schon von ANTLR importiert:
	import java.util.List;
	import java.util.ArrayList;
	import java.util.HashMap;
	import java.util.Map;
	*/
}

@members
{
	// ********** Log4j Logger of this class **********
	public static final Logger LOG = Logger.getLogger(GroundAtomWalker.class);
    	// ************************************************

	private boolean initialized = false;
	private LanguageFactory lang = null;
	//private VariableFactory varFac = null; // a new VariableFactory is instantiated for each clause and accessed via this global variable
	private ArrayList<Query> queryList = null;
	private ArrayList<AtomExpression> atomExpList = null;

    	/**
    	 * This Method MUST be called before starting the walker with the method <code>queryInput</code>.
    	 */
    	public void init(LanguageFactory existingLangFac)
    	{
    		this.lang = existingLangFac;
    		initialized = true;
    	}

	public List<Query> getQueries()
	{
		return queryList;
	}

	public List<AtomExpression> getGroundAtoms()
	{
		return atomExpList;
	}

}



// $<Query

queriesInput
	@init
	{
		if (initialized == false)
		{
			throw new RuntimeException("The method 'init' must be called before starting the walker.");
		}
		queryList = new ArrayList<Query>();
	}
	:	^(QUERIES (query
				{
					queryList.add($query.value);
				}
			   )+
		)
	;
	
groundAtomsInput
	@init
	{
		if (initialized == false)
		{
			throw new RuntimeException("The method 'init' must be called before starting the walker.");
		}
	}
	:	groundAtomExpressions
		{
			atomExpList = $groundAtomExpressions.valueList; // not really neccessary, because atomExpList should already be assigned in groundAtomExpressions  
		}
	;

query		returns[Query value]
		@init
		{
			// for each query, a new VariableFactory is instantiated
			//varFac = new VariableFactory();
		}
	:	^(QUERY questionPart groundAtomExpressions?)
		{			
			/* $value = new Query($query.text, lang, $questionPart.valueList, $groundAtomExpressions.valueList, varFac); */
			$value = new Query($query.text, lang, $questionPart.valueList, $groundAtomExpressions.valueList);
		}
	;

questionPart	returns[ArrayList<AtomExpression> valueList]
		@init
		{
			$valueList = new ArrayList<AtomExpression>();
		}
	:	^(QUESTION_PART (atomExpression
				{$valueList.add($atomExpression.value);}
			      )+
		)
	;

groundAtomExpressions
		returns[ArrayList<AtomExpression> valueList]
		@init
		{
			$valueList = new ArrayList<AtomExpression>();
			atomExpList = $valueList; // make sure atomExpList always references an initiated list (whether or not groundAtomsInput was called)
		}
	:	^(GROUND_ATOM_EXPRESSIONS EMPTY) // empty file
	|	^(GROUND_ATOM_EXPRESSIONS (atomExpression
					{$valueList.add($atomExpression.value);}
				)+
		)
	;

atomExpression	returns[AtomExpression value]
	:	^(ATOM_EXPRESSION atom rangeValue)
		{
			// TODO: LogicalRange sorgfaelltiger checken (so wie in GroundAtomWithoutLanguageWalker)
			$value = new AtomExpression($atom.value, $rangeValue.value);
		}
	|	^(TRUE_ATOM_EXPRESSION atom)
		{
			$value = new AtomExpression($atom.value, TruthValue.TRUE.toString());
		}
	|	^(FALSE_ATOM_EXPRESSION atom)
		{
			$value = new AtomExpression($atom.value, TruthValue.FALSE.toString());
		}
	;

// $>



// $<Logic Elements
atom		returns[GeneralizedAtom value]
	:	^(ATOM predicate arguments[$predicate.value]?)
		{
			GeneralizedPredicate pred = $predicate.value;
			
			ArrayList<Term> argList = ($arguments.valueList == null) ? new ArrayList<Term>() : $arguments.valueList;

			// check number of arguments
			if (pred.getArity() != argList.size())
			{
				throw new ParsingRuntimeException($predicate.line, $predicate.pos,
				"The predicate '" + pred + "' must not have " + argList.size() + " arguments.",
				ParsingErrorInfo.ErrorType.SEMANTIC);
			}

			// construct the atom
			$value = new GeneralizedAtom(pred, argList);
		}
	;

predicate	returns[GeneralizedPredicate value, int line, int pos]
	:	^(PREDICATE identifierL)
		{	
			GeneralizedPredicate pred = lang.getExistingPredicate($identifierL.value);	

			// check if predicate has been defined afore
			if (pred == null)
			{
				throw new ParsingRuntimeException($identifierL.line, $identifierL.pos,
				"The predicate '" + $identifierL.value + "' has not been defined before.",
				ParsingErrorInfo.ErrorType.SEMANTIC);
			}
			
			$value = pred;
			$line  = $identifierL.line;
			$pos   = $identifierL.pos;
		}
	;

arguments[GeneralizedPredicate pred]	
	returns[ArrayList<Term> valueList]
	@init
	{
		$valueList = new ArrayList<Term>();
		int i = 0;
	}
	:	^(ARGUMENTS (term[pred.getArgSort(i)]
				  {
				  	$valueList.add($term.value);
				  	i++;
				  }
			    )+
		)
	;

term[Sort sort]		
	returns[Term value]
	:	variable[sort]
		{
			$value = $variable.value;
		}
	|	constant[sort]
		{
			$value = $constant.value;
		}
	;

//arity		returns[int value]
//	:	^(ARITY DIGITS)
//		{
//			$value = Integer.parseInt($DIGITS.text);
//		}
//	;

rangeValue	returns[String value]
	:	^(RANGE_VALUE identifierL)
		{
			$value = new String($identifierL.value);
		}
	;

constant[Sort sort]
	returns[Constant value]
	:	^(CONSTANT identifierL)
		{
			String constantString = $identifierL.value;
			
			if (lang.getGrammarStyle() == LanguageFactory.GrammarStyle.UPPER_CASE_CONSTS_AND_LOWER_CASE_VARS)
			{
				// change the first character of the constant to upper-case 
				String origStr = new String(constantString);
				String firstChar = constantString.substring(0, 1).toUpperCase();
				String otherChar = constantString.substring(1);
				constantString = firstChar + otherChar;
				LOG.debug("Constant converted to upper-case: '" + origStr + "' --> '" + constantString + "'");
			}
			
			try
			{
				// If the constant is not defined at this point, it is added to the LanguageFactory.
				$value = Constant.createOrGet(lang, constantString, $sort);
			}
			catch (WriteProtectionException ex)
			{
				LOG.error(ex.getMessage());
				throw new RuntimeException(ex);
			}
		}
	;

variable[Sort sort]
	returns[Variable value]
	:	^(VARIABLE identifierU)
		{
			String variableString = $identifierU.value;
			
			if (lang.getGrammarStyle() == LanguageFactory.GrammarStyle.UPPER_CASE_CONSTS_AND_LOWER_CASE_VARS)
			{
				// change the first character of the variable to lower-case 
				String origStr = new String(variableString);
				String firstChar = variableString.substring(0, 1).toLowerCase();
				String otherChar = variableString.substring(1);
				variableString = firstChar + otherChar;
				LOG.debug("Variable converted to lower-case: '" + origStr + "' --> '" + variableString + "'");
			}
			
			throw new ParsingRuntimeException($identifierU.line, $identifierU.pos,
			"Variable '" + variableString + "' found: Variables must not occur in queries or ground atom expressions.",
				ParsingErrorInfo.ErrorType.SEMANTIC);
			//$value = varFac.getVariable(variableString, $sort);
		}
	;

// $>



// $<Identifiers and Keywords

identifierL returns [String value, int line, int pos]
	:	str=IDENTIFIER_L
		{
			$value = new String($str.text);
			$line  = $str.line;
			$pos   = $str.pos;
		}
	;


identifierU returns [String value, int line, int pos]
	:	str=IDENTIFIER_U
		{
			$value = new String($str.text);
			$line = $str.line;
			$pos = $str.pos;
		}
	;

// $>


